/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.usercontrols;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.PNestedDisplayBean.PNestedDisplayBean;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.util.ShrimpFileFilter;
import edu.umd.cs.piccolo.PCanvas;

/**
 * Exports the current view to an image file.
 * 
 * @author Rob Lintern
 **/
public class ExportToImageFileAction extends DefaultProjectAction {
	
	private String toolName;
	private boolean jpegAvailable = false;
	private boolean pngAvailable = false;
	private boolean gifAvailable = false;
	private FileFilter jpegFileFilter = null;
	private FileFilter pngFileFilter = null;
	private FileFilter gifFileFilter = null;
	
	public ExportToImageFileAction(ShrimpProject project, String toolName) {
		super("Export to Image File...", project);
		this.toolName = toolName;
	}

	public void startAction() {
		String [] formats = ImageIO.getWriterFormatNames();
		String allFormats = "";
		for (int i = 0; i < formats.length; i++) {
			String format = formats[i];
			if (format.toLowerCase().equals("jpeg") || format.toLowerCase().equals("jpg")) {
				jpegAvailable = true;
				allFormats += "JPG ";
			} else if (format.toLowerCase().equals("png")) {
				pngAvailable = true;
				allFormats += "PNG ";
			} else if (format.toLowerCase().equals("gif")) {
				gifAvailable = true;
				allFormats += "GIF";
			}
		}
		JFileChooser chooser = new JFileChooser();
		chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		
		chooser.setDialogTitle("Export to Image File (" + allFormats + ")");
		chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
		//chooser.setAcceptAllFileFilterUsed(true);
		
		if (jpegAvailable) {
			jpegFileFilter = new ShrimpFileFilter (new String [] {"jpg", "jpeg"}, "JPEG Images");
			chooser.addChoosableFileFilter(jpegFileFilter);
		}
		if (pngAvailable) {
			pngFileFilter = new ShrimpFileFilter (new String [] {"png"}, "PNG Images");
			chooser.addChoosableFileFilter(pngFileFilter);
		}
		if (gifAvailable) {
			gifFileFilter = new ShrimpFileFilter (new String [] {"gif"}, "GIF Images");
			chooser.addChoosableFileFilter(gifFileFilter);
		}
		
		int state = chooser.showSaveDialog(ApplicationAccessor.getApplication().getParentFrame());
		if (state == JFileChooser.APPROVE_OPTION) {
			File file = chooser.getSelectedFile();
			FileFilter fileFilter = chooser.getFileFilter();
			String filePath = file.toString();
			String formatName = "";
			if (jpegFileFilter != null && jpegFileFilter.equals(fileFilter)) {
				formatName = "jpg";
				if (!filePath.toLowerCase().endsWith(".jpg") && !filePath.toLowerCase().endsWith(".jpeg")) {
					filePath += ".jpg";
					file = new File (filePath);
				}
			}
			if (pngFileFilter != null && pngFileFilter.equals(fileFilter)) {
				formatName = "png";
				if (!filePath.toLowerCase().endsWith(".png")) {
					filePath += ".png";
					file = new File (filePath);
				}
			}
			if (gifFileFilter != null && gifFileFilter.equals(fileFilter)) {
				formatName = "gif";
				if (!filePath.toLowerCase().endsWith(".gif")) {
					filePath += ".gif";
					file = new File (filePath);
				}
			}
			boolean saveImage = false;
			if (file.exists()) {
				int opt = JOptionPane.showConfirmDialog(ApplicationAccessor.getApplication().getParentFrame(), "The file " + file.toString() + " already exists. Push OK to overwrite this existing file.", "Export Warning", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
				saveImage = opt == JOptionPane.OK_OPTION;
			} else {
				try {
					file.createNewFile();
					saveImage = true;
				} catch (IOException e) {
					e.printStackTrace();
					JOptionPane.showMessageDialog(ApplicationAccessor.getApplication().getParentFrame(), "There was a problem creating " + file.toString() + ". The image has not been exported.", "Export Error", JOptionPane.ERROR_MESSAGE);
				}
			}
			
			if (saveImage) {
				saveImage (file, formatName);
			}
		}			
	}
	
	private void saveImage(File file, String formatName) {
		if (hasProject()) {
			try {
				ShrimpTool tool = getTool(toolName);
				DisplayBean displayBean = (DisplayBean) tool.getBean(ShrimpTool.DISPLAY_BEAN);
				if (displayBean instanceof PNestedDisplayBean) {
					PCanvas canvas = ((PNestedDisplayBean)displayBean).getPCanvas();
					int width = canvas.getWidth();
					int height = canvas.getHeight();
					BufferedImage bufferedImage = new BufferedImage (width, height, BufferedImage.TYPE_INT_RGB);
					Graphics g = bufferedImage.getGraphics();
					canvas.paint(g); // paint the canvas to the image
					
					ImageIO.write(bufferedImage, formatName, file);
					
					JOptionPane.showMessageDialog(ApplicationAccessor.getApplication().getParentFrame(), file.toString() + " has been saved successfully.");
				}
			} catch (Exception e) {
				JOptionPane.showMessageDialog(ApplicationAccessor.getApplication().getParentFrame(), "There was a problem saving " + file.toString() + ". The image has not been exported.", "Export Error", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			} 
		}
	}

}
